﻿using System;

using UIKit;
using Infragistics;
using Foundation;
using System.Collections.Generic;

namespace CalendarAppointments_CS
{
	public class CalendarDataSource : IGCalendarAppointmentDataSource
	{
		NSObject[] _appts;
		public CalendarDataSource(NSObject[] appointments)
		{
			_appts = appointments;
		}

		public override NSObject[] GetAppointments (IGCalendarView calView, NSDate start, NSDate end, IGCalendarAppointmentRequestType requestType)
		{
			List<NSObject> returnAppts = new List<NSObject>();

			double min = start.SecondsSinceReferenceDate;
			double max = end.SecondsSinceReferenceDate;

			foreach(NSDictionary appt in _appts)
			{
				NSDate startTime = appt["start"] as NSDate;
				double current = startTime.SecondsSinceReferenceDate;

				if(current >= min && current <= max)
				{
					IGCalendarAppointment calAppt = new IGCalendarAppointment();
					calAppt.StartTime = startTime;
					calAppt.EndTime = appt[@"end"] as NSDate;
					calAppt.Location = appt[@"location"].ToString();
					calAppt.Title = appt[@"title"].ToString();

					returnAppts.Add(calAppt);
				}
			}

			return returnAppts.ToArray ();
		}

		public override UIColor ResolveAppointmentColor (IGCalendarView calView)
		{
			return UIColor.Red;
		}
	}

	public partial class ViewController : UIViewController
	{
		NSObject[] _appts;
		IGCalendarView _calendar;
		CalendarDataSource _dataSource;
		NSObject[] _dataSources;

		public ViewController (IntPtr handle) : base (handle)
		{
		}

		public override void ViewDidLoad ()
		{
			base.ViewDidLoad ();

			_appts = GenerateAppointments (500);
			_calendar = new IGCalendarView ();
			_calendar.Frame = this.View.Bounds;
			_calendar.AutoresizingMask = UIViewAutoresizing.FlexibleWidth|UIViewAutoresizing.FlexibleHeight;
			_calendar.MinYear = 2016;
			_calendar.MaxYear = 2016;
			this.View.Add (_calendar);

			_dataSource = new CalendarDataSource (_appts);
			_dataSources = new NSObject[] { _dataSource };
			_calendar.AppointmentDataSources = _dataSources;
		}

		public override void DidReceiveMemoryWarning ()
		{
			base.DidReceiveMemoryWarning ();
		}

		public override bool PrefersStatusBarHidden ()
		{
			return true;
		}

		public NSObject[] GenerateAppointments(int count)
		{
			Random rnd = new Random (DateTime.Now.Millisecond);
			string dateString, month, day, year, time, location, title;
			string[] titles = new string[] { "Webinar About", "Conference Covering", "Meeting About", "Meeting With" };
			string[] subjects = new string[] { "Apple", "Development", "Engineering", "iOS", "Marketing", "NucliOS", "Sales" };
			string[] locations = new string[] { "Head Quarters", "London, UK", "Montevideo, UY", "Tokoyo, JP" };
			List<NSObject> appointments = new List<NSObject>();

			for (int j = 0; j < count; j++)
			{
				NSMutableDictionary appointment = new NSMutableDictionary();

				month = string.Format ("{0}", rnd.Next (11) + 1);
				day = string.Format ("{0}", rnd.Next (27) + 1);
				year = "2016";
				time = string.Format ("{0}:{1}{2}", rnd.Next (23), rnd.Next (5), 0);

				location = string.Format ("{0}", locations [rnd.Next (locations.Length)]);
				title = string.Format("{0} {1}", titles[rnd.Next(titles.Length)], subjects[rnd.Next(subjects.Length)]);

				dateString = string.Format("{0}-{1}-{2} {3}", month, day, year, time);
				DateTime parsedDate = DateTime.Parse (dateString);
				NSDate startDate = DateTimeToNSDate(parsedDate);
				NSDate endDate = DateTimeToNSDate(parsedDate.AddMinutes (rnd.Next (15, 90)));

				appointment.SetValueForKey (startDate, new NSString("start"));
				appointment.SetValueForKey (endDate, new NSString("end"));
				appointment.SetValueForKey (new NSString(title), new NSString("title"));
				appointment.SetValueForKey (new NSString(location), new NSString("location"));

				appointments.Add (appointment);
			}

			return appointments.ToArray();
		}

		public static NSDate DateTimeToNSDate(DateTime date)
		{
			DateTime reference = TimeZone.CurrentTimeZone.ToLocalTime(
				new DateTime(2001, 1, 1, 0, 0, 0) );
			return NSDate.FromTimeIntervalSinceReferenceDate(
				(date - reference).TotalSeconds);
		}
	}
}

